<?php

namespace SuperRhino\Utils\Common;

/**
 * Class Hash
 * @package SuperRhino\Utils
 *
 */
class PasswordHash {

    /**
     * The default cost factor.
     *
     * @var int
     */
    protected static $rounds = 10;

    /**
     * Hash the given value.
     * @param string $value
     * @param array $options
     * @return string
     */
    public static function make(string $value, array $options = []) {
        $hash = password_hash($value, PASSWORD_BCRYPT, [
            'cost' => self::cost($options),
        ]);

        if ($hash === false) {
            throw new \RuntimeException('Bcrypt hashing not supported.');
        }

        return $hash;
    }

    /**
     * @param string $value
     * @param string $hashedValue
     * @param array $options
     * @return bool
     */
    public static function check(string $value, string $hashedValue, array $options = []) {
        if (self::info($hashedValue)['algoName'] !== 'bcrypt') {
            throw new \RuntimeException('This password does not use the Bcrypt algorithm.');
        }

        if (strlen($hashedValue) === 0) {
            return false;
        }

        return password_verify($value, $hashedValue);
    }

    /**
     * @param string $hashedValue
     * @return array
     */
    public static function info(string $hashedValue): array {
        return password_get_info($hashedValue);
    }

    /**
     * Extract the cost value from the options array.
     *
     * @param array $options
     * @return int
     */
    protected static function cost(array $options = []) {
        return $options['rounds'] ?? self::$rounds;
    }
}